package RASAgentSNMPTrap;

###############################################################
#
#  This package is for generating an SNMP Version 2 trap
#  that is a NOTIFICATION_TYPE defined by the StorageAgent.mib
#  MIB file.  (OID 1.3.6.1.4.1.42.2.88.0.6) 
#
#  The following attributes must be set on a RASAgentSNMPTrap
#  object, before sending the trap using the sendTrap method:
#
#    Attribute        Description
#    -------------    --------------------------------------- 
#    origIPAddr       IP Address of the host sending trap. 
#    destHostname     The Hostname of machine accpetin traps. 
#    destPort         Port on the host receiving traps. 
#    deviceName       Device that the alert message pertains. 
#    alertLevel       Alert Level related to the message. 
#    message          The actual alert message. 
# 
###############################################################

use BER '0.58';
use SNMP_Session '0.58'; 


   # Static attributes of RASAgentSNMPTrap include:
   # 
   #   Attribute        Description
   #   -------------    ---------------------------------------
   #   trapOID          OID for Storage Agent trap definition. 
   #   deviceNameOID    OID for the device name attribute.  
   #   alertLevelOID    OID for the alert level attribute. 
   #   messageOID       OID for the alert message attribute. 
   #   trapCommunity    The community the trap is intended for. 

   my @trapOID = ( 1,3,6,1,4,1,42,2,88,0,6 );
   my @deviceNameOID = ( 1,3,6,1,4,1,42,2,88,1,3,1 );
   my @alertLevelOID = ( 1,3,6,1,4,1,42,2,88,1,3,2 );
   my @messageOID = ( 1,3,6,1,4,1,42,2,88,1,3,3);
   my $trapCommunity = 'public';


   #
   # Instantiate SNMPTrap object.
   #
   # Usage:
   #  $trap = RASAgentSNMPTrap->new({   IP => "1.2.3.4",
   #                                  host => "hostname",
   #                                  port => 162,
   #                                device => "Purple12",
   #                            alertLevel => 2 });
   #
   #  $trap->setMessage('CRASHING Dieter...');
   #  $trap->sendTrap();
 
   sub new {
      my($class, $arg) = @_;
      my $this = {};
      bless $this , "RASAgentSNMPTrap";

      if ($arg) {
         if ($arg->{IP}) {
            my(@a) = split(/\./, $arg->{IP});
             $this->setOrigIPAddr($a[0], $a[1],$a[2],$a[3]);
         } else {
            Debug->err(TEXT => "Must specify IP in trap->new()");
            return undef;
         }
         if ($arg->{host} && $arg->{port}) {
           $this->setDestHostname($arg->{host});
           $this->setDestPort($arg->{port});
         } else {
           Debug->err(TEXT => "Must specify HostName and Port in trap->new()");
           return undef;
         }
         $this->setDeviceName($arg->{device});
         $this->setAlertLevel($arg->{alertLevel} || 1);
      }
      return $this;
   }


   #
   # Set origIPAddr  
   # Parameters:  
   #     $val1 - Integer representing first numer in IP Address 
   #     $val2 - Integer representing second number in IP Address
   #     $val3 - Integer representing third number in IP Address
   #     $val4 - Integer representing last number in IP Address 
   #
   sub setOrigIPAddr {
      my ($this, $val1, $val2, $val3, $val4) = @_;
      return if (!$val1);
      $this->{'origIPAddr'} = pack "CCC", $val1, $val2, $val3, $val4;
      return;
   }

   #
   # Get origIPAddr
   #
   sub getOrigIPAddr {
       my ($this) = @_;
       return $this->{'origIPAddr'};
   }


   #
   # Set destHostname
   # Parameters:
   #    $val - String for trap receiving hostname 
   #
   sub setDestHostname {
      my ($this, $val) = @_;
      return if (!$val);
      $this->{'destHostname'} = $val;
      return;
   } 

   #
   # Get destHostname 
   #
   sub getDestHostname {
       my ($this) = @_;
       return $this->{'destHostname'};
   }


   #
   # Set destPort
   # Parameters:
   #    $val - Integer for port host is receiving traps on 
   #
   sub setDestPort {
      my ($this, $val) = @_;
      return if (!$val);
      $this->{'destPort'} = $val;
      return;
   }

   #
   # Get destPort 
   #
   sub getDestPort {
       my ($this) = @_;
       return $this->{'destPort'};
   }
   

   #
   # Set deviceName
   # Parameters:
   #    $val - String for the device name that the alert pertains to 
   #
   sub setDeviceName {
      my ($this, $val) = @_;
      return if (!$val);
      $this->{'deviceName'} = $val;
      return;
   }

   #
   # Get deviceName 
   #
   sub getDeviceName {
       my ($this) = @_;
       return $this->{'deviceName'};
   }


   #
   # Set alertLevel
   # Parameters:
   #    $val - String for the alert level for the alert 
   #
   sub setAlertLevel {
      my ($this, $val) = @_;
      return if (!$val);
      $this->{'alertLevel'} = $val;
      return;
   }

   #
   # Get alertLevel 
   #
   sub getAlertLevel {
       my ($this) = @_;
       return $this->{'alertLevel'};
   }

 
   #
   # Set message
   # Parameters:
   #    $val - String for alert message 
   #
   sub setMessage {
      my ($this, $val) = @_;
      return if (!$val);
      $this->{'message'} = $val;
      return;
   } 

   #
   # Get message 
   #
   sub getMessage {
       my ($this) = @_;
       return $this->{'message'};
   }

 
   #
   # Send the trap created to the host and port specified. 
   #
   sub sendTrap {

      # First Check to ensure all the required values are set.

      my $startTime = time; 
      # Open up trap session.
      my ($this) = @_; 
      my $trapSession = SNMPv2c_Session->open ( $this->{'destHostname'}, $trapCommunity, $this->{'destPort'});
 
      # Send the trap. 
      my $upTime = int ((time - $startTime ) * 100); 
      $trapSession->v2_trap_request_send (\@trapOID, $upTime,
					[encode_oid (@deviceNameOID),
					 encode_string($this->{'deviceName'})],
                                        [encode_oid (@alertLevelOID),
                                         encode_int ($this->{'alertLevel'})],
                                        [encode_oid (@messageOID),
                                         encode_string ($this->{'message'})]);

} 
 
1;
